C      PROGRAM SPF6.FOR BY MIKE FOLSE   LAST MODIFIED 2/23/96
C      LINEAR STATIC ANALYSIS OF SPACE FRAMES
C       USE CONSISTENT UNITS
C     
C               - NODAL RENUMBERING
C               - LINEAR STORAGE OF STIFFNESS MATRIX
C
C      INPUT FROM FILE SPF6.DAT OR FROM KEYBOARD
C       YOU MAY BUILD SPF6.DAT IN AN EDITOR OR WITHIN PROGRAM
C       PROGRAM AUTOMATICALLY STORES KEYBOARD INPUT IN SPF6.DAT
C      OUTPUT IS TO FILE SPF6.OUT
C
C      INPUT LIST FOR FILE SPF6.DAT(FREE FORMAT-SEPARATE INPUT
C           ITEMS WITH ONE COMMA OR ONE OR MORE BLANKS,INPUT
C           ALL ITEMS EVEN IF ZERO)
C
C      LINE 1: TITLE(15A4)
C      LINE 2:
C              NUMBER OF JOINTS(NJT),
C              NUMBER OF JOINTS WITH APPLIED LOADS(NLDJT),
C              NUMBER OF JOINT X TRANSLATIONS SPECIFIED(NXTS)
C              NUMBER OF JOINT Y TRANSLATIONS SPECIFIED(NYTS)
C              NUMBER OF JOINT Z TRANSLATIONS SPECIFIED(NZTS)
C              JOINT NUMBER ON WHICH TO BEGIN JOINT RENUMBERING
C                   FRONTAL METHOD USED
C                 = NEGATIVE  -  NO RENUMBERING
C                 = 0 OR .GT.NJT  - DEFAULT TO JOINT 1
C                  (RENUMBERING ONLY USED IF RESULTING NDIAG IS
C                    SMALLER THAN FOR INPUT NUMBERING)
C      LINE 3:
C              NUMBER OF MEMBERS(NMEM),
C              NUMBER OF MEMBER TYPES(NMTYPE),
C              NUMBER OF MEMBERS WITH END RELEASES (NMWR)
C               (NMWR=-1 INDICATES TRUSS)
C              MODULUS OF ELASTICITY(E),
C              MODULUS OF RIGIDITY(G),
C
C      LINE 4 (REPEAT NJT TIMES):
C              JOINT NUMBER
C              X COORDINATE,
C              Y COORDINATE,
C              Z COORDINATE,
C              SUPPORT CODE(NS)
C     (NS=1 IF AT LEAST 1 DOF AT JOINT IS SUPPORT,0 OTHERWISE)
C         NOTE:INPUT JOINTS IN ASCENDING ORDER,MISSING JOINTS WILL
C               BE GENERATED BASED ON EQUAL SPACING
C      LINE 4A  IF NS=1,INPUT SPRING SUPPORT CONSTANTS FOR
C                 X TRANSLATION
C                 Y TRANSLATION 
C                 Z TRANSLATION
C                 X ROTATION
C                 Y ROTATION
C                 Z ROTATION
C            IF NEGATIVE NUMBER INPUT,DEFAULT = 1.E30 ASSIGNED
C
C      LINE 5 (REPEAT NMTYPE TIMES):
C              MEMBER TYPE NUMBER
C              CROSS SECTION AREA
C              MOMENT OF INERTIA ABOUT X AXIS (POLAR)
C              MOMENT OF INERTIA ABOUT Y AXIS (MINOR)
C              MOMENT OF INERTIA ABOUT Z AXIS (MAJOR)
C
C
C      LINE 6 (REPEAT NMEM TIMES):
C              MEMBER NUMBER
C              JOINT NUMBER OF JOINT TO BE CONSIDERED J NODE 
C              JOINT NUMBER OF K NODE 
C              JOINT NUMBER OF L NODE DEFINING XY PLANE OF MEMBER,
C                (FOR W SHAPE, XY PLANE CONTAINS THE WEB)
C                    L = -1 : XL = XJ +1000, YL = YJ, ZL = ZJ
C                    L = -2 : XL = XJ, YL = YJ + 1000,ZL = ZJ
C                    L = -3 : XL = XJ, YL = YJ, ZL = ZJ + 1000
C              MEMBER TYPE NUMBER
C              MEMBER LOAD CODE(MLC)
C               (=1 IF MEMBER HAS LOAD, =0 OTHERWISE)
C      LINE 6A   IF MLC=1, INPUT 
C              VALUE OF DISTRIBUTED Y LOAD AT J NODE,
C              VALUE OF DISTRIBUTED Y LOAD AT K NODE
C              THETA ANGLE IN DEGREES CCW (LOOKING FROM POSITIVE
C           LOCAL Z SIDE) FROM PERPENDICULAR FOR DISTRIBUTED Y LOAD
C              VALUE OF Z LOAD AT J NODE,
C              VALUE OF Z LOAD AT K NODE,
C              THETA ANGLE IN DEGREES CCW (LOOKING FROM POSITIVE
C                  LOCAL Y SIDE) FROM PERPENDICULAR
C                 FOR DISTRIBUTED Z LOAD
C              NUMBER OF CONCENTRATED Y LOADS,(MUST BE PERPEND)
C              NUMBER OF CONCENTRATED Z LOADS      "
C                 (3 MAX EACH DIRECTION)
C               INPUT POSITIVE LOAD IN POSITIVE LOCAL DIRECTION
C      LINE 6B LOAD VALUE,DISTANCE FROM JNODE FOR Y LOADS,IF ANY
C      LINE 6C LOAD VALUE,DISTANCE FROM J NODE FOR Z LOADS,IF ANY
C      LINE 7 (REPEAT NLDJT TIMES): 
C                  JOINT NUMBER OF LOADED JOINT,
C                  APPLIED LOAD IN X DIRECTION,
C                  APPLIED LOAD IN Y DIRECTION,
C                  APPLIED LOAD IN Z DIRECTION,
C                  APPLIED CCW MOMENT ABOUT X AXIS,
C                  APPLIED CCW MOMENT ABOUT Y AXIS  
C                  APPLIED CCW MOMENT ABOUT Z AXIS
C      LINE 8  (REPEAT NMWR TIMES):
C                  MEMBER NUMBER OF MEMBER WITH RELEASES
C           RELEASE CODE FOR BENDING ABOUT Z AXIS(MAJOR) (NPZ)
C                      = 0  NO RELEASES
C                      = 1  PINNED BOTH ENDS
C                      = 2  PINNED AT J NODE
C                      = 3  PINNED AT K NODE
C           RELEASE CODE FOR BENDING ABOUT Y AXIS(MINOR) (NPY)
C                      = 0 NO RELEASES
C                      = 1 PINNED BOTH ENDS
C                      = 2 PINNED AT J NODE
C                      = 3 PINNED AT K NODE
C           RELEASE CODE FOR AXIAL LOAD AND TORSION  (NPX)
C                      = 0 NO RELEASES
C                      = 1 TORSION RELEASE, NO AXIAL RELEASE
C                      = 2 TORSION RELEASE, AXIAL RELEASE AT J
C                      = 3 TORSION RELEASE, AXIAL RELEASE AT K
C                      = 4 NO TORSION RELEASE,AXIAL RELEASE AT J
C                      = 5 NO TORSION RELEASE,AXIAL RELEASE AT K
C    LINE 9  (REPEAT NXTS TIMES):(10MAX)
C                   JOINT NUMBER WITH SPECIFIED X TRANSLATION
C                   SPECIFIED X TRANSLATION
C    LINE 10  (REPEAT NYTS TIMES):(10 MAX)
C                   JOINT NUMBER WITH SPECIFIED Y TRANSLATION
C                   SPECIFIED Y TRANSLATION
C    LINE 11 (REPEAT NZTS TIMES):(10MAX)
C                   JOINT NUMBER WITH SPECIFIED Z TRANSLATION
C                   SPECIFIED Z TRANSLATION
       CALL INPUT
       CALL ANALYSIS
       CALL OUTPUT
       STOP
       END
C
C
       SUBROUTINE INPUT
       DIMENSION TITLE(15)
       COMMON/JJ/AXT(20),RIXT(20),RIYT(20),RIZT(20) 
       COMMON/HH/IOUT,NJT,NMEM,E,G,NMWR,NXTS,NYTS,NZTS
       COMMON/GG/D(12),INN(6),SM(12,12),
     1 SUP(400,6),FC(6),SMR(12,12),IN(12),SMRT(12,12),
     1 EACT(12),NS(400),MLC(600),NPX(600),NPY(600),NPZ(600),
     1 WLY(600),WRY(600),WLZ(600),WRZ(600),FEM(12),P(12),
     1 NCLDY(600),NCLDZ(600),PV(600,6),PD(600,6),MTP(600),
     1 THTDY(600),THTDZ(600)
      COMMON/AA/X(400),Y(400),Z(400),JEND(600),KEND(600),LEND(600),
     1 BJT(2400),NJXT(10),SXT(10),NJYT(10),SYT(10),NJZT(10),SZT(10)
      COMMON/BB/CX(600),CY(600),CZ(600),CX2(600),CY2(600),CZ2(600),
     1CX3(600),CY3(600),CZ3(600),ALEN(600),RT(12,12),RTRANS(12,12)
      COMMON/CC/A(70000),B(2400),IDIAG(2400),KLIM(2400),IB,NDIAG
      COMMON/KK/NN(400)
C     NN(I) IS NEW NODE NUMBER FOR INPUT NODE I
      CHARACTER CDIR*1
      OPEN(6,FILE='SPF6.OUT',STATUS='UNKNOWN')
C      PROGRAM DIMENSIONED FOR MAXIMUM 400 JOINTS AND 600 MEMBERS
      JDIM=70000
      NCD=0
      IB=0     
      INP=5
      IOUT=6
      IP=7
      WRITE(*,5000)
 5000 FORMAT(5X,'SELECT INPUT MODE',//,15X,'(1)  KEYBOARD',
     1//,15X,'(0)  FILE SPF6.DAT',//,15X,'(2)  TERMINATE',/)
      READ(*,*)NCD
      IF(NCD.NE.0.AND.NCD.NE.1)STOP
      WRITE(*,5005)
 5005 FORMAT(/,5X,'OUTPUT IS TO FILE SPF6.OUT',//,5X,
     1 ' "EDIT SPF6.OUT" TO VIEW CONTENTS',//,5X,
     1 ' "PRINT SPF6.OUT" TO GET PRINTED OUTPUT')
      IF(NCD.EQ.1)GO TO 50
      OPEN(5,FILE='SPF6.DAT',STATUS='OLD',ACCESS='SEQUENTIAL')
      READ(INP,1000)TITLE
      GO TO 60
   50 CONTINUE
      OPEN(7,FILE='SPF6.DAT',STATUS='UNKNOWN')
      WRITE(*,5010)
 5010 FORMAT(/,5X,'ENTER TITLE UP TO 60 CHARACTERS',/)
      READ(*,1000)TITLE
      WRITE(IP,1001)TITLE
 1001 FORMAT(1X,15A4)
 1000 FORMAT(15A4)
   60 WRITE(IOUT,2000)TITLE
 2000 FORMAT(//,10X,15A4)
      IF(NCD.EQ.1)GO TO 80
      READ(INP,*)NJT,NLDJT,NXTS,NYTS,NZTS,NST
      READ(INP,*)NMEM,NMTYPE,NMWR,E,G
      GO TO 85
   80 WRITE(*,5020)
 5020 FORMAT(/,5X,'ENTER NUMBER OF JOINTS(400 MAXIMUM)',/)
      READ(*,*)NJT
      WRITE(*,5022)
 5022 FORMAT(/,5X,'ENTER NUMBER OF JOINTS WITH APPLIED LOADING',/)
      READ(*,*)NLDJT
      WRITE(*,7022)
 7022 FORMAT(/,5X,'ENTER NUMBER OF JOINTS WITH SPECIFIED',
     1' X TRANSLATIONS',/)
      READ(*,*)NXTS
      WRITE(*,7023)
 7023 FORMAT(/,5X,'ENTER NUMBER OF JOINTS WITH SPECIFIED',
     1' Y TRANSLATIONS',/)
      READ(*,*)NYTS
      WRITE(*,7024)
 7024 FORMAT(/,5X,'ENTER NUMBER OF JOINTS WITH SPECIFIED',
     1' Z TRANSLATIONS',/)
      READ(*,*)NZTS
      WRITE(*,7027)
 7027 FORMAT(/,5X,'ENTER JOINT NUMBER WHERE RENUMBERING TO BEGIN',
     1/,10X,'IF NEGATIVE INTEGER INPUT, NO RENUMBERING',/)
      READ(*,*)NST
      WRITE(*,5021)
 5021 FORMAT(/,5X,'ENTER NUMBER OF MEMBERS (600 MAXIMUM)',/)
      READ(*,*)NMEM
      WRITE(*,7025)
 7025 FORMAT(/,5X,'ENTER NUMBER OF MEMBER TYPES (20 MAXIMUM)',/)
      READ(*,*)NMTYPE
      WRITE(*,7026)
 7026 FORMAT(/,5X,'ENTER NUMBER OF MEMBERS WITH END RELEASES',
     1 /,10X,'-1 INDICATES SPACE TRUSS',/)
      READ(*,*)NMWR
      WRITE(*,5023)
 5023 FORMAT(/,5X,'ENTER MODULUS OF ELASTICITY (USE CONSISTENT',
     1 ' UNITS)',/)
      READ(*,*)E
      WRITE(*,5026)
 5026 FORMAT(/,5X,'ENTER MODULUS OF RIGIDITY G',/)
      READ(*,*)G
      WRITE(IP,1010)NJT,NLDJT,NXTS,NYTS,NZTS,NST
      WRITE(IP,1011)NMEM,NMTYPE,NMWR,E,G
 1010 FORMAT(8I5)
 1011 FORMAT(3I5,2E12.3)
      WRITE(*,5024)
 5024 FORMAT(///,5X,'INPUT JOINT COORDINATES AND SPRING ',
     1 'SUPPORTS',//)
   85 WRITE(IOUT,2010)NJT,NLDJT,NXTS,NYTS,NZTS,NST,NMEM,
     1 NMTYPE,NMWR,E,G
 2010 FORMAT(//,5X,'NUMBER OF JOINTS =',I5,
     1 /,5X,'NUMBER OF JOINTS WITH APPLIED LOAD(S) =',I5,
     1 /,5X,'NUMBER OF JOINTS WITH SPECIFIED X TRANSLATION =',I5,
     1 /,5X,'NUMBER OF JOINTS WITH SPECIFIED Y TRANSLATION =',I5,
     1 /,5X,'NUMBER OF JOINTS WITH SPECIFIED Z TRANSLATION =',I5,
     1 /,5X,'JOINT AT WHICH TO BEGIN RENUMBERING =',I5,
     1 /,5X,'NUMBER OF MEMBERS =',I5,
     1 /,5X,'NUMBER OF MEMBER TYPES =',I5,
     1 /,5X,'NUMBER OF MEMBERS WITH END RELEASES =',I5,
     1 /,10X,'(-1 INDICATES SPACE TRUSS)',
     1/,5X,'MODULUS OF ELASTICITY =',E12.3,
     1/,5X,'MODULUS OF RIGIDITY G =',E12.3,
     1 //,3X,'JOINT       X           Y         Z')
      NUMEQ=6*NJT
      DO 87 I=1,NJT
      DO 87 J=1,6
   87 SUP(I,J)=0.
      MM=0
   88 CONTINUE
      IF(NCD.EQ.1)GO TO 90
      READ(INP,*)M,X(M),Y(M),Z(M),NS(M)
      IF(NS(M).EQ.1)READ(INP,*)(SUP(M,J),J=1,6)
      GO TO 100
   90 WRITE(*,5030)  
 5030 FORMAT(/,2X,'ENTER JT NUMBER,X COORD,Y COORD,Z COORD',
     1' AND SUPPORT CODE(=1 IF ANY SUPPORTS)',/)
      READ(*,*)M,X(M),Y(M),Z(M),NS(M)
      IF(NS(M).EQ.1)WRITE(*,5031)
 5031 FORMAT(/,2X,'ENTER SPRING SUPPORT CONSTANTS X,Y,Z',
     1' TRANSLATION AND X,Y,Z ROTATION',/,5X,
     1 ' -1 INDICATES COMPLETE FIXITY AT ANY GIVEN DOF',/)
      IF(NS(M).EQ.1)READ(*,*)(SUP(M,J),J=1,6)
      WRITE(IP,1020)M,X(M),Y(M),Z(M),NS(M)
 1020 FORMAT(I5,3E12.3,I5)
      IF(NS(M).EQ.1)WRITE(IP,1021)(SUP(M,J),J=1,6)
 1021 FORMAT(6E12.3)
  100 CONTINUE
      DO 105 J=1,6
  105 IF(SUP(M,J).LT.0.)SUP(M,J)=1.E30
      IF(M.EQ.(MM+1))GO TO 116
C     GENERATE MISSING NODES
      M1=MM+1
      M2=M-1
      DIFF=FLOAT(M-MM)
      XINC=(X(M)-X(MM))/DIFF
      YINC=(Y(M)-Y(MM))/DIFF
      ZINC=(Z(M)-Z(MM))/DIFF
      AINC=0.
      DO 115 K=M1,M2
      AINC=AINC+1.
      X(K)=X(MM)+AINC*XINC
      Y(K)=Y(MM)+AINC*YINC
      Z(K)=Z(MM)+AINC*ZINC
C     GENERATED NODES CANNOT BE SUPPORTS
      NS(K)=0
  115 CONTINUE
  116 IF(M.EQ.NJT)GO TO 119
      MM=M
      GO TO 88
  119 CONTINUE
      DO 120 I=1,NJT
  120 WRITE(IOUT,2020)I,X(I),Y(I),Z(I)
 2020 FORMAT(1X,I5,5E12.3)
      WRITE(IOUT,2021)
 2021 FORMAT(//,5X,'SPRING CONSTANTS',/,3X,'JT      SX',
     110X,'SY',10X,'SZ',10X,'SRX',9X,'SRY',9X,'SRZ')
      DO 125 I=1,NJT
      IF(NS(I).EQ.0)GO TO 125
      WRITE(IOUT,2022)I,(SUP(I,J),J=1,6)
 2022 FORMAT(I5,6E12.3)
  125 CONTINUE
      DO 650 I =1,NMTYPE
      IF(NCD.EQ.1)GO TO 600
      READ(INP,*)M,AXT(M),RIXT(M),RIYT(M),RIZT(M)
      GO TO 630
  600 CONTINUE
      WRITE(*,6040)
 6040 FORMAT(/,2X,'ENTER MEMBER TYPE,  AREA,  X INERTIA(POLAR),',
     1/,20X,'Y INERTIA(MINOR),   Z INERTIA(MAJOR)',//)
      READ(*,*)M,AXT(M),RIXT(M),RIYT(M),RIZT(M)
      WRITE(IP,6030)M,AXT(M),RIXT(M),RIYT(M),RIZT(M)
 6030 FORMAT(I5,4E12.3)
  630 CONTINUE
C     OUTPUT MEMBER TYPE INFORMATION
      IF(I.EQ.1)WRITE(IOUT,6050)
 6050 FORMAT(/,1X,'MEMBER TYPE    AREA      XINERTIA(POLAR)',
     1'  YINERTIA(MINOR)  ZINERTIA(MAJOR)',/)
      WRITE(IOUT,6060)M,AXT(M),RIXT(M),RIYT(M),RIZT(M)
 6060 FORMAT(5X,I5,2X,E10.2,3(6X,E10.2))
  650 CONTINUE
      DO 140 I=1,NMEM
      NPX(I)=0
      NPY(I)=0
      NPZ(I)=0
      IF(NCD.EQ.1)GO TO 130
      READ(INP,*)M,JEND(M),KEND(M),LEND(M),MTP(M),MLC(M)
      IF(MLC(M).EQ.0)GO TO 135
      READ(INP,*)WLY(M),WRY(M),THTDY(M),
     1 WLZ(M),WRZ(M),THTDZ(M),NCLDY(M),NCLDZ(M)
      NCL=NCLDY(M)
      IF(NCL.GT.0)READ(INP,*)(PV(M,J),PD(M,J),J=1,NCL)
      NCL=NCLDZ(M)+3
      IF(NCL.GT.3)READ(INP,*)(PV(M,J),PD(M,J),J=4,NCL)
      GO TO 135
  130 CONTINUE
      WRITE(*,5040)
 5040 FORMAT(/,1X,'ENTER MEMBER NUMBER,J NODE,K NODE,L NODE',
     1',MEM TYPE, LOAD CODE',
     1/,5X,'(PLANE JKL IS LOCAL XY PLANE, +Y TOWARD L HALFPLANE)',
     1/,5X,'L=-1 : XL = XJ + 1000, YL = YJ, ZL = ZJ',/,
     1 5X,'L = -2 : XL = XJ, YL = YJ + 1000, ZL = ZJ',/,
     1 5X,'L = -3 : XL = XJ, YL = YJ, ZL = ZJ + 1000',/,
     1 8X,'(LOAD CODE =1 IF ANY MEMBER LOADS, 0 OTHERWISE)',/)
      READ(*,*)M,JEND(M),KEND(M),LEND(M),MTP(M),MLC(M)
      WRITE(IP,1030)M,JEND(M),KEND(M),LEND(M),MTP(M),MLC(M)
 1030 FORMAT(6I5)
      IF(MLC(M).EQ.0)GO TO 135
      CDIR='Y'
      WRITE(*,5042)CDIR
 5042 FORMAT(/,2X,'ENTER ',A1,
     1 ' DISTRIBUTED LOAD VALUE J END AND K END',/,
     1 5X,'(INPUT LOADS IN POSITIVE LOCAL DIRECTION AS POSITIVE)',
     1 /,5X,'AND CCW ANGLE (DEGREES) FROM PERPENDICULAR TO LOAD',
     1/,5X,' DIRECTION (LOOK FROM +Z OR +Y)',/)
      READ(*,*)WLY(M),WRY(M),THTDY(M)
      CDIR='Z'
      WRITE(*,5042)CDIR
      READ(*,*)WLZ(M),WRZ(M),THTDZ(M)
      WRITE(*,7043)
 7043 FORMAT(/,2X,'ENTER NUMBER OF CONCENTRATED LOADS IN Y DIR',
     1' AND Z DIR (3 MAX EACH DIR)',/,5X,
     1 '(CONCENTRATED LOADS ARE PERPENDICULAR TO MEMBER)',/)
      READ(*,*)NCLDY(M),NCLDZ(M)
      WRITE(IP,7044)WLY(M),WRY(M),THTDY(M),WLZ(M),WRZ(M),THTDZ(M),
     1 NCLDY(M),NCLDZ(M)
 7044 FORMAT(6E10.2,2I5)
      NCL=NCLDY(M)
      CDIR='Y'
      IF(NCL.GT.0)WRITE(*,7045)CDIR
 7045 FORMAT(/,2X,'ENTER (LOAD VALUE,DISTANCE FROM J) FOR EACH',
     1 1X,A1,' LOAD',/)
      IF(NCL.GT.0)READ(*,*)(PV(M,J),PD(M,J),J=1,NCL)
      IF(NCL.GT.0)WRITE(IP,1021)(PV(M,J),PD(M,J),J=1,NCL)
      NCL=NCLDZ(M)+3
      CDIR='Z'
      IF(NCL.GT.3)WRITE(*,7045)CDIR
      IF(NCL.GT.3)READ(*,*)(PV(M,J),PD(M,J),J=4,NCL)
      IF(NCL.GT.3)WRITE(IP,1021)(PV(M,J),PD(M,J),J=4,NCL)
  135 CONTINUE
C
  140 CONTINUE
C     CALL SUBROUTINE TO RENUMBER NODES
      CALL RENUM(NST)
C     WRITE OUT NEW NODE NUMBERING
      WRITE(IOUT,8333)(I,NN(I),I=1,NJT)
 8333 FORMAT(5X,'NODE ',I5,' BECOMES NODE ',I5)
C     CALCULATE BAND WIDTH OF STIFFNESS MATRIX
      DO 142 I = 1,NMEM
      NBW=6*(IABS(NN(JEND(I))-NN(KEND(I)))+1)
      IF(NBW.GT.IB)IB=NBW
  142 CONTINUE
      WRITE(IOUT,2025)IB
 2025 FORMAT(//,10X,'BAND WIDTH SIZE',
     1 'IB=',I5)
C     COMPUTE KLIM,IDIAG,NDIAG.
C     NDIAG IS THE NUMBER OF NONZERO TERMS IN THE STIFFNESS MATRIX
C     KLIM(I) IS THE COLUMN HEIGHT IN STIFFNESS MATRIX FOR I'TH DOF
C IDIAG(I) IS THE POSITION IN THE ONE DIMENSIONAL STIFFNESS MATRIX
C     OCCUPIED BY THE DIAGONAL OF THE I'TH DOF. COLUMN NUMBERING
C     IS FROM THE TOP DOWN TO THE DIAGONAL 
      CALL KL(NUMEQ,NMEM,JDIM)
      WRITE(IOUT,2030)
 2030 FORMAT(//,3X,'MEMBER  J NODE   K NODE   L NODE  MEMBER TYPE',
     1 '  LENGTH',/)
      DO 160 I=1,NMEM
      JJ=JEND(I)
      KK=KEND(I)
      XC=X(KK)-X(JJ)
      YC=Y(KK)-Y(JJ)
      ZC=Z(KK)-Z(JJ)
      AL=SQRT(XC*XC+YC*YC+ZC*ZC)
  160 WRITE(IOUT,2040)I,JEND(I),KEND(I),LEND(I),MTP(I),AL
 2040 FORMAT(1X,5I8,5X,E12.3)
      CDIR='Y'
      WRITE(IOUT,2035)CDIR
 2035 FORMAT(//,10X,'MEMBER LOADS IN LOCAL ',A1,' DIRECTION',/,
     1 2X,'MEMBER    DISTRIB LD VAL AT J   VAL AT K',
     1' CCW ANGL FRM +Y OR Z,DEG  P       D',/)
      DO 170 I=1,NMEM
      IF(MLC(I).EQ.0)GO TO 170
      NCL=NCLDY(I)
      WRITE(IOUT,2041)I,WLY(I),WRY(I),THTDY(I),
     1 (PV(I,J),PD(I,J),J=1,NCL)
 2041 FORMAT(1X,I4,9X,E12.3,5X,2E12.3,(/,55X,2E10.2))
  170 CONTINUE
      CDIR='Z'
      WRITE(IOUT,2035)CDIR
      DO 175 I=1,NMEM
      IF(MLC(I).EQ.0)GO TO 175
      NCL=NCLDZ(I)+3
      WRITE(IOUT,2041)I,WLZ(I),WRZ(I),THTDZ(I),
     1 (PV(I,J),PD(I,J),J=4,NCL)
  175 CONTINUE
C     INITIALIZE STIFFNESS AND LOAD MATRICES TO ZERO
      DO 180 I=1,NUMEQ
      B(I)=0.
      BJT(I)=0.
  180 CONTINUE
      DO 4000 I = 1,NDIAG
 4000 A(I) = 0.
      IF(NLDJT.EQ.0)GO TO 191
C     INPUT JOINT LOADS
      WRITE(IOUT,2045)
 2045 FORMAT(//,2X,'JOINT  XFORCE    YFORCE    ZFORCE',
     15X,'XMOM       YMOM       ZMOM',/)
      DO 190 I=1,NLDJT
      IF(NCD.EQ.1)GO TO 185
      READ(INP,*)JT,(FC(J),J=1,6)
      GO TO 188
  185 CONTINUE
      WRITE(*,5050)
 5050 FORMAT(/,5X,'ENTER JOINT,FORCE X,Y,Z DIRECTIONS,',
     1 'AND CC MOMENT X,Y,Z AXES',/)
      READ(*,*)JT,(FC(J),J=1,6)
      WRITE(IP,2022)JT,(FC(J),J=1,6)
  188 CONTINUE
      WRITE(IOUT,2050)JT,(FC(J),J=1,6)
 2050 FORMAT(1X,I4,6E11.3)
C     ADD FORCES TO APPLIED LOAD MATRIX
      CALL IND(NN(JT),INN)
      DO 190 J=1,6
      B(INN(J))=B(INN(J))+FC(J)
      BJT(INN(J))=BJT(INN(J))+FC(J)
  190 CONTINUE
  191 CONTINUE
      IF(NMWR.LE.0)GO TO 210
      DO 207 I=1,NMWR
      IF(NCD.EQ.1)GO TO 200
      READ(INP,*)MEM,NPZ(MEM),NPY(MEM),NPX(MEM)
      GO TO 205
  200 CONTINUE
      WRITE(*,2051)
 2051 FORMAT(/,2X,'ENTER MEMBER NUMBER AND RELEASE CODE FOR',
     1 ' STRONG AXIS BENDING',
     1 /,12X,'0 = NO RELEASES, 1 = PINNED BOTH ENDS',
     1 /,12X,'2 = PINNED J END ONLY, 3 = PINNED K END ONLY',/)
      READ(*,*)MEM,NPZ(MEM)
      WRITE(*,2052)
 2052 FORMAT(/,2X,'ENTER RELEASE CODE FOR WEAK AXIS BENDING',/)
      READ(*,*)NPY(MEM)
      WRITE(*,2053)
 2053 FORMAT(/,2X,'ENTER AXIAL/TORSIONAL RELEASE CODE',/,
     1 3X,'0 = NO RELEASES',/,
     1 3X,'1 = TORSION RELEASE ONLY',/,
     1 3X,'2 = TORSION RELEASE PLUS AXIAL RELEASE J END',/,
     1 3X,'3 = TORSION RELEASE PLUS AXIAL RELEASE K END',/,
     1 3X,'4 = NO TORSION RELEASE, AXIAL RELEASE J END',/,
     1 3X,'5 = NO TORSION RELEASE, AXIAL RELEASE K END',/)
      READ(*,*)NPX(MEM)
      WRITE(IP,5055)MEM,NPZ(MEM),NPY(MEM),NPX(MEM)
 5055 FORMAT(4I5)
  205 CONTINUE
      IF(I.EQ.1)WRITE(IOUT,5056)
 5056 FORMAT(//,22X,'MEMBER RELEASE CODES',/,2X,
     1 'MEMBER    Z AXIS BENDING(ST)    Y AXIS BENDING(WK)',
     1 5X,'AXIAL/TORSIONAL',/)
      WRITE(IOUT,5057)MEM,NPZ(MEM),NPY(MEM),NPX(MEM)
 5057 FORMAT(2X,I5,3(13X,I5))
  207 CONTINUE
  210 CONTINUE
      IF(NXTS.EQ.0)GO TO 710
      CDIR='X'
      IF(NCD.EQ.1)GO TO 700
      READ(INP,*)(NJXT(I),SXT(I),I=1,NXTS)
      GO TO 705
  700 CONTINUE
      DO 701 I=1,NXTS
      WRITE(*,7051)CDIR
 7051 FORMAT(/,5X,'ENTER JOINT NUMBER AND SPECIFIED ',A1,
     1' TRANSLATION',/)
      READ(*,*)NJXT(I),SXT(I)
      WRITE(IP,7055)NJXT(I),SXT(I)
 7055 FORMAT(I5,E12.3)
  701 CONTINUE
  705 CONTINUE
      WRITE(IOUT,7056)CDIR
 7056 FORMAT(//,5X,'JOINT    SPECIFIED ',A1,' TRANSLATION',/)
      DO 707 I=1,NXTS
      WRITE(IOUT,7057)NJXT(I),SXT(I)
 7057 FORMAT(5X,I5,10X,E12.3)
  707 CONTINUE
  710 CONTINUE
      IF(NYTS.EQ.0)GO TO 810
      CDIR='Y'
      IF(NCD.EQ.1)GO TO 800
      READ(INP,*)(NJYT(I),SYT(I),I=1,NYTS)
      GO TO 805
  800 CONTINUE
      DO 801 I=1,NYTS
      WRITE(*,7051)CDIR
      READ(*,*)NJYT(I),SYT(I)
      WRITE(IP,7055)NJYT(I),SYT(I)
  801 CONTINUE
  805 CONTINUE
      WRITE(IOUT,7056)CDIR
      DO 807 I=1,NYTS
      WRITE(IOUT,7057)NJYT(I),SYT(I)
  807 CONTINUE
  810 CONTINUE
      IF(NZTS.EQ.0)GO TO 910
      CDIR='Z'
      IF(NCD.EQ.1)GO TO 900
      READ(INP,*)(NJZT(I),SZT(I),I=1,NZTS)
      GO TO 905
  900 CONTINUE
      DO 901 I=1,NZTS
      WRITE(*,7051)CDIR
      READ(*,*)NJZT(I),SZT(I)
      WRITE(IP,7055)NJZT(I),SZT(I)
  901 CONTINUE
  905 CONTINUE
      WRITE(IOUT,7056)CDIR
      DO 907 I=1,NZTS
      WRITE(IOUT,7057)NJZT(I),SZT(I)
  907 CONTINUE
  910 CONTINUE
      RETURN
      END
C

      SUBROUTINE RENUM(NST)
C     SUBROUTINE TO RENUMBER THE JOINTS FOR BETTER STORAGE
C       EFFICIENCY
       DIMENSION NFLS(200),NFLSN(200),NNSTAT(400)
       COMMON/RR/NNT(400,6)
       COMMON/HH/IOUT,NJT,NMEM,E,G,NMWR,NXTS,NYTS,NZTS
      COMMON/AA/X(400),Y(400),Z(400),JEND(600),KEND(600),LEND(600),
     1 BJT(2400),NJXT(10),SXT(10),NJYT(10),SYT(10),NJZT(10),SZT(10)
      COMMON/KK/NN(400)
      N6=NJT*6
      IF(NST.LT.0) GO TO 500
      IF(NST.EQ.0.OR.NST.GT.NJT)NST=1
      K=1
    5 CONTINUE
      DO 10 I = 1,NJT
      NNSTAT(I) = 0
   10 CONTINUE
      NNT(NST,K)=1
      NFLS(1)=NST
      NNSTAT(NST)=1
      NNNF=1
C     NFLS IS LIST OF NODES IN CURRENT NODEFRONT
C     NNSTAT = 0 INDICATES NODE NOT YET RENUMBERED
C            = 1 INDICATES NODE ALREADY RENUMBERED
C     NST IS NODE TO START RENUMBERING ON
C     NNSUM IS NEXT NEW NODE NUMBER ASSIGNED
C     NNNF IS NUMBER OF NODES IN CURRENT NODEFRONT
      NNSUM=2
   20 CONTINUE
      INF=0
      DO 50 I=1,NNNF
      ND=NFLS(I)
      DO 40 J=1,NMEM
      JJ=JEND(J)
      KK=KEND(J)
      IF(JJ.NE.ND.AND.KK.NE.ND)GO TO 40
      IF(JJ.NE.ND)GO TO 30
C     JJ .EQ. ND  --- RUNUMBER KK AND PUT IN NEW NODEFRONT IF IT
C       HAS NOT ALREADY BEEN RENUMBERED
      IF(NNSTAT(KK).EQ.1)GO TO 40
      NNT(KK,K)=NNSUM
      NNSUM=NNSUM+1
      NNSTAT(KK)=1
      INF=INF+1
      NFLSN(INF)=KK
      GO TO 40
   30 CONTINUE
C     KK .EQ. ND  ---- RENUMBER JJ AND PUT IN NEW NODEFRONT IF IT
C       HAS NOT ALREADY BEEN RENUMBERED
      IF(NNSTAT(JJ).EQ.1)GO TO 40
      NNT(JJ,K)=NNSUM
      NNSUM=NNSUM+1
      NNSTAT(JJ)=1
      INF=INF+1
      NFLSN(INF)=JJ
   40 CONTINUE
   50 CONTINUE
C     CREATE NEW NODEFRONT ARRAY
      IF(INF.EQ.0)GO TO 80
      NNNF=INF
      DO 60 I=1,NNNF
   60 NFLS(I) = NFLSN(I)
      GO TO 20
   80 CONTINUE
C     ASSIGN NUMBERS TO NODES NOT ATTACHED TO ANY MEMBERS
      DO 100 I= 1,NJT
      IF(NNSTAT(I).EQ.1)GO TO 100
      NNT(I,K)=NNSUM
      NNSUM=NNSUM+1
  100 CONTINUE
C     DETERMINE NDIAG FOR NEW NUMBERING
      CALL KL2(N6,NMEM,1,NDGT1)
      WRITE(IOUT,1000)NDGT1
 1000 FORMAT(/,2X,'ELEMENTS IN STIFFNESS MATRIX FOR',
     1 ' NEW NUMBERING =',I8)
C     FOR K = 2 USE ORIGINAL NUMBERING
      K=2
      DO 120 I = 1,NJT
  120 NNT(I,K)=I
      CALL KL2(N6,NMEM,2,NDGT2)
      WRITE(IOUT,2000)NDGT2
 2000 FORMAT(/,2X,'ELEMENTS IN STIFFNESS MATRIX FOR',
     1 ' OLD NUMBERING =',I8)
      IF(NDGT2.GT.NDGT1)K=1
      DO 140 I=1,NJT
      NN(I)=NNT(I,K)
  140 CONTINUE
      RETURN
  500 CONTINUE
      DO 550 I=1,NJT
  550 NN(I)=I
      RETURN
      END
      SUBROUTINE KL2(N6,NMEM,KKK,NDIAG)
      COMMON/RR/NNT(400,6)
      COMMON/AA/X(400),Y(400),Z(400),JEND(600),KEND(600),LEND(600),
     1 BJT(2400),NJXT(10),SXT(10),NJYT(10),SYT(10),NJZT(10),SZT(10)
      DIMENSION IN(12),KLIM(2400)
C     SUBROUTINE TO COMPUTE NDIAG FOR A GIVEN NODE NUMBERING
C     LOOP THROUGH ALL MEMBERS, COMPUTE PSUEDO 
C         INDEX VECTOR FOR EACH MEMBER WITH J .LT. K
C     COMPUTE COLUMN HEIGHT TO SKYLINE.
C     COLUMN ENTRIES NUMBERED FROM TOP DOWN TO DIAGONAL
      DO 10 I = 1,N6
  10  KLIM(I) = 0
      DO 20 M = 1,NMEM
      J = NNT(JEND(M),KKK)
      K = NNT(KEND(M),KKK)
      IF(J .LT. K) GO TO 25
      J = K
      K = NNT(JEND(M),KKK)
C
  25  J6 = 6*J
      DO 200 I = 1,6
      J = 7-I
  200 IN(J) = J6-I+1
      K6 = 6*K
      DO 300 I = 1,6
      J = 13-I
  300 IN(J) = K6-I+1
C
C     COMPUTE COLUMN HEIGHTS FOR MTH MEMBER.
C
      DO 30 L = 1,12
      N = IN(L)
      IH = IN(L)-IN(1)+1
      IF(KLIM(N).GT.IH)GO TO 30
      KLIM(N) = IH
  30  CONTINUE
  20  CONTINUE
      NDIAG = 0
      DO 40 I = 1,N6
      NDIAG = NDIAG+KLIM(I)
  40  CONTINUE
  50  RETURN
      END
C
      SUBROUTINE ANALYSIS
      COMMON/AA/X(400),Y(400),Z(400),JEND(600),KEND(600),LEND(600),
     1 BJT(2400),NJXT(10),SXT(10),NJYT(10),SYT(10),NJZT(10),SZT(10)
      COMMON/BB/CX(600),CY(600),CZ(600),CX2(600),CY2(600),CZ2(600),
     1CX3(600),CY3(600),CZ3(600),ALEN(600),RT(12,12),RTRANS(12,12)
      COMMON/CC/A(70000),B(2400),IDIAG(2400),KLIM(2400),IB,NDIAG
      COMMON/JJ/AXT(20),RIXT(20),RIYT(20),RIZT(20) 
      COMMON/HH/IOUT,NJT,NMEM,E,G,NMWR,NXTS,NYTS,NZTS
      COMMON/GG/D(12),INN(6),SM(12,12),
     1 SUP(400,6),FC(6),SMR(12,12),IN(12),SMRT(12,12),
     1 EACT(12),NS(400),MLC(600),NPX(600),NPY(600),NPZ(600),
     1 WLY(600),WRY(600),WLZ(600),WRZ(600),FEM(12),P(12),
     1 NCLDY(600),NCLDZ(600),PV(600,6),PD(600,6),MTP(600),
     1 THTDY(600),THTDZ(600)
      COMMON/KK/NN(400)
      NUMEQ=6*NJT
      LU=1
C     FORM MEMBER STIFFNESS MATRICES, ROTATE TO GLOBAL COORDINATES,
C       INDEX TO GLOBAL DEGREE OF FREEDOM NUMBERS, AND ADD TO
C       GLOBAL STIFFNESS MATRIX
      DO 300 I=1,NMEM
      IF(NMWR.LT.0)NPZ(I)=1
      IF(NMWR.LT.0)NPY(I)=1
      JJ=JEND(I)
      KK=KEND(I)
      CALL DIRCOS(I)
      AL=ALEN(I)
      CALL FORMRT(I)
      CALL FORMSM(SM,AL,AXT(MTP(I)),E,G,RIXT(MTP(I)),
     1 RIYT(MTP(I)),RIZT(MTP(I)),NPX(I),NPY(I),NPZ(I))
C      OPTION TO PRINT STIFFNESS MATRICES NO LONGER GIVEN
C      IF(NPR.NE.1)GO TO 230
C      IF(I.EQ.1)WRITE(IOUT,8200)
C 8200 FORMAT(/,10X,'MEMBER STIFFNESS MATRICES IN LOCAL COORD')
C      WRITE(IOUT,8205)I,AL
C 8205 FORMAT(//,10X,'MEMBER',I5,'  LENGTH =',E12.3)
C      DO 220 J=1,12
C  220 WRITE(IOUT,8210)(SM(J,K),K=1,12)
C 8210 FORMAT(/,(1X,6E12.3))
C  230 CONTINUE
      CALL MULT(SM,RT,12,SMRT)
      CALL MULT(RTRANS,SMRT,12,SMR)
      CALL INDX(NN(JJ),NN(KK),IN)
C     STORE UPPER HALF OF STIFFNESS MATRIX IN LINEAR ARRAY
C     NUMBERING OF ELEMENTS IN COLUMN IS FROM TOP DOWN
      DO 260 J=1,12
      JG=IN(J)
      DO 260 K=1,12
      KG=IN(K)
      IF(KG.LT.JG)GO TO 260
      IS = IDIAG(KG) - KG + JG
      A(IS)=A(IS)+SMR(J,K)
  260 CONTINUE
C     ADD MEMBER END ACTIONS TO LOAD MATRIX
      IF(MLC(I).EQ.0)GO TO 300
      CALL FM(WLY(I),WRY(I),THTDY(I),WLZ(I),WRZ(I),
     1 THTDZ(I),AL,FEM,NPX(I),NPY(I),NPZ(I))
C      CALL FM2 AS REQD
      NCL=NCLDY(I)
      IF(NCL.EQ.0)GO TO 263
      DO 262 J=1,NCL
  262 CALL FM2(PV(I,J),PD(I,J),AL,FEM,NPZ(I),0)
  263 CONTINUE
      NCL=NCLDZ(I)
      IF(NCL.EQ.0)GO TO 265
      DO 264 J=1,NCL
      JP3=J+3
  264 CALL FM2(PV(I,JP3),PD(I,JP3),AL,FEM,NPY(I),1)
  265 CALL MULT(RTRANS,FEM,1,P)
C     STORE FEM IN PV AND PD MATRICES
      DO 266 J=1,6
      JP6=J+6
      PV(I,J)=FEM(J)
      PD(I,J)=FEM(JP6)
  266 CONTINUE
      DO 270 J=1,12
      B(IN(J))=B(IN(J))-P(J)
  270 CONTINUE
  300 CONTINUE
C     CORRECT GLOBAL STIFFNESS MATRIX FOR BOUNDARY CONDITIONS
C       I.E. SUPPORT SPRINGS WHERE INPUT
      DO 360 I=1,NJT
      CALL IND(NN(I),INN)
      IF(NS(I).EQ.0)GO TO 350
      DO 345 J=1,6
      IS = IDIAG(INN(J))
  345 A(IS)=A(IS)+SUP(I,J)
  350 CONTINUE
      IF(NMWR.GE.0)GO TO 360
C     FOR TRUSS, FIX NODES AGAINST ROTATION
      IS = IDIAG(INN(4))
      A(IS)=1.E30
      IS= IDIAG(INN(5))
      A(IS)=1.E30
      IS = IDIAG(INN(6))
      A(IS)=1.E30
  360 CONTINUE
C      WRITE(IOUT,7100)
C 7100 FORMAT(//,10X,'STRUCTURE STIFFNESS MATRIX',
C     1' (linear storage)')
C      DO 3650 I=1,NDIAG
C 3650 WRITE(IOUT,8000)I,A(I)
C 8000 FORMAT(10X,'I = ',I5,'  A(I) = ',E12.3)
C      WRITE(IOUT,7200)
C 7200 FORMAT(//,10X,'APPLIED LOAD MATRIX')
C      WRITE(IOUT,8100)(I,B(I),I=1,NUMEQ)
C 8100 FORMAT(4(1X,I4,E12.3))
C
C      CORRECT FOR SPECIFIED TRANSLATIONS
      IF(NXTS.EQ.0) GO TO 362
      DO 361 I=1,NXTS
      NDOF=6*NN(NJXT(I))-5
      IS=IDIAG(NDOF)
      A(IS)=1.E28
      B(NDOF)=1.E28*SXT(I)
  361 CONTINUE
  362 CONTINUE
      IF(NYTS.EQ.0) GO TO 364
      DO 363 I=1,NYTS
      NDOF=6*NN(NJYT(I))-4
      IS=IDIAG(NDOF)
      A(IS)=1.E28
      B(NDOF)=1.E28*SYT(I)
  363 CONTINUE
  364 CONTINUE
      IF(NZTS.EQ.0) GO TO 366
      DO 365 I=1,NZTS
      NDOF=6*NN(NJZT(I))-3
      IS=IDIAG(NDOF)
      A(IS)=1.E28
      B(NDOF)=1.E28*SZT(I)
  365 CONTINUE
  366 CONTINUE 
C     SOLVE   AX=B FOR DISPLACEMENT MATRIX X
      CALL UDU2(LU,NUMEQ)
C      IF(NER.EQ.0)GO TO 370
C     WRITE(IOUT,4200)
C4200 FORMAT(/,10X,'STIFFNESS MATRIX IS SINGULAR')
C     STOP
C 370 CONTINUE
C     NOTE: DEFLECTIONS ARE RETURNED IN B MATRIX WHICH WENT INTO
C         UDU AS APPLIED LOADS
C     
      RETURN
      END
C
C
      SUBROUTINE OUTPUT
      DIMENSION SUM(2400),DL(12),DLS(600,12)
      COMMON/AA/X(400),Y(400),Z(400),JEND(600),KEND(600),LEND(600),
     1 BJT(2400),NJXT(10),SXT(10),NJYT(10),SYT(10),NJZT(10),SZT(10)
      COMMON/BB/CX(600),CY(600),CZ(600),CX2(600),CY2(600),CZ2(600),
     1CX3(600),CY3(600),CZ3(600),ALEN(600),RT(12,12),RTRANS(12,12)
      COMMON/CC/A(70000),B(2400),IDIAG(2400),KLIM(2400),IB,NDIAG
      COMMON/JJ/AXT(20),RIXT(20),RIYT(20),RIZT(20) 
      COMMON/HH/IOUT,NJT,NMEM,E,G,NMWR,NXTS,NYTS,NZTS
      COMMON/GG/D(12),INN(6),SM(12,12),
     1 SUP(400,6),FC(6),SMR(12,12),IN(12),SMRT(12,12),
     1 EACT(12),NS(400),MLC(600),NPX(600),NPY(600),NPZ(600),
     1 WLY(600),WRY(600),WLZ(600),WRZ(600),FEM(12),P(12),
     1 NCLDY(600),NCLDZ(600),PV(600,6),PD(600,6),MTP(600),
     1 THTDY(600),THTDZ(600)
      COMMON/KK/NN(400)
      NUMEQ=6*NJT
      DO 100 I=1,NUMEQ
      SUM(I)=0.
  100 CONTINUE
  370 WRITE(IOUT,4100)
 4100 FORMAT(//,2X,'OUTPUT',//,10X,'JOINT DISPLACEMENTS ',
     1 //,5X,'JT '
     1,' X TRANS   Y TRANS    Z TRANS      X ROT    Y ROT',
     16X,'Z ROT')
      DO 400 I=1,NJT
      CALL IND(NN(I),INN)
      WRITE(IOUT,4110)I,(B(INN(J)),J=1,6)
 4110 FORMAT(1X,I5,6E11.3)
  400 CONTINUE
      WRITE(IOUT,4115)
 4115 FORMAT(//,13X,'MEMBER END ACTIONS',//,1X,'MEM  NODE ',
     1'AXIAL      SHEAR Y     SHEAR Z    TORSION   MOMENT Y',
     13X,'MOMENT Z',/)
      DO 500 I=1,NMEM
      AL=ALEN(I)
      JJ=JEND(I)
      KK=KEND(I)
      CALL INDX(NN(JJ),NN(KK),IN)
      DO 450 J=1,12
  450 D(J)=B(IN(J))
      CALL FORMRT(I)
      CALL FORMSM(SM,AL,AXT(MTP(I)),E,G,RIXT(MTP(I)),
     1 RIYT(MTP(I)),RIZT(MTP(I)),NPX(I),NPY(I),NPZ(I))
      CALL MULT(RT,D,1,DL)
      DO 455 J=1,12
      DLS(I,J)=DL(J)
  455 CONTINUE
      CALL MULT(SM,DL,1,EACT)
C     ADD FIXED END ACTIONS
      IF(MLC(I).EQ.0)GO TO 480
      DO 460 K=1,6
      KP6=K+6
      EACT(K)=EACT(K)+PV(I,K)
      EACT(KP6)=EACT(KP6)+PD(I,K)
  460 CONTINUE
  480 WRITE(IOUT,4120)I,JJ,(EACT(J),J=1,6)
      WRITE(IOUT,4120)I,KK,(EACT(J),J=7,12)
 4120 FORMAT(1X,2I4,6E11.3)
C     ROTATE END ACTIONS TO GLOBAL COORDINATES FOR RECREATION
C      OF JOINT LOADS AND REACTIONS
      CALL MULT(RTRANS,EACT,1,P)
      DO 482 J=1,12
      SUM(IN(J))=SUM(IN(J))+P(J)
  482 CONTINUE
  500 CONTINUE
C     OUTPUT MEMBER DEFORMATIONS IN LOCAL COORDINATES
      WRITE(IOUT,4210)
 4210 FORMAT(//,2X,'MEMBER DEFORMATIONS IN LOCAL COORDINATES',/,
     15X,'(-0.999E+22 INDICATES DEFORMATION NOT COMPUTED DUE',
     1' TO END RELEASES)',//,'  MEM  NODE    XT       YT    ',
     15X,'ZT         XR         YR         ZR',/)
      CON=-0.999E+22
      DO 600 I=1,NMEM
      NPMX=NPX(I)
      NPMY=NPY(I)
      NPMZ=NPZ(I)
      JJ=JEND(I)
      KK=KEND(I)
      IF(NPMX.LE.3.AND.NPMX.GT.0)DLS(I,10)=CON
      IF(NPMX.EQ.2.OR.NPMX.EQ.4)DLS(I,1)=CON
      IF(NPMX.EQ.3.OR.NPMX.EQ.5)DLS(I,7)=CON
      IF(NPMY.EQ.1.OR.NPMY.EQ.2)DLS(I,5)=CON
      IF(NPMY.EQ.1.OR.NPMY.EQ.3)DLS(I,11)=CON
      IF(NPMZ.EQ.1.OR.NPMZ.EQ.2)DLS(I,6)=CON
      IF(NPMZ.EQ.1.OR.NPMZ.EQ.3)DLS(I,12)=CON
      WRITE(IOUT,4215)I,JJ,(DLS(I,J),J=1,6)
      WRITE(IOUT,4215)I,KK,(DLS(I,J),J=7,12)
 4215 FORMAT(1X,2I4,6E11.3)
  600 CONTINUE
C     SUBTRACT JOINT LOAD MATRIX FROM SUMMATION OF MEMBER END
C        ACTIONS
      DO 710 I=1,NUMEQ
      SUM(I)=SUM(I)-BJT(I)
  710 CONTINUE
C     OUTPUT REACTIONS
      WRITE(IOUT,4230)
 4230 FORMAT(//,5X,'REACTIONS RECREATED FROM'
     1' MEMBER END ACTIONS',
     1/,5X,'JT   XFOR     YFOR',
     17X,'ZFOR       XMOM       YMOM       ZMOM',/)
      DO 720 I=1,NJT
      CALL IND(NN(I),INN)
      WRITE(IOUT,4110)I,(SUM(INN(J)),J=1,6)
  720 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE DIRCOS(I)
      COMMON/AA/X(400),Y(400),Z(400),JEND(600),KEND(600),LEND(600),
     1 BJT(2400),NJXT(10),SXT(10),NJYT(10),SYT(10),NJZT(10),SZT(10)
      COMMON/BB/CX(600),CY(600),CZ(600),CX2(600),CY2(600),CZ2(600),
     1CX3(600),CY3(600),CZ3(600),ALEN(600),RT(12,12),RTRANS(12,12)
      JJ=JEND(I)
      KK=KEND(I)
      LL=LEND(I)
      IF(LL.GT.0)GO TO 10
      IF(LL.EQ.-1)GO TO 20
      IF(LL.EQ.-2)GO TO 30
C     LL = -3  XL = XJ, YL = YJ, ZL = ZJ +1000.
      XLL = X(JJ)
      YLL = Y(JJ)
      ZLL = Z(JJ) + 1000.
      GO TO 40
   10 CONTINUE
C     LL .GT. 0
      XLL = X(LL)
      YLL = Y(LL)
      ZLL = Z(LL)
      GO TO 40
   20 CONTINUE
C     LL = -1
      XLL = X(JJ) + 1000.
      YLL = Y(JJ)
      ZLL = Z(JJ)
      GO TO 40
   30 CONTINUE
C     LL = -2
      XLL = X(JJ)
      YLL = Y(JJ) + 1000.
      ZLL = Z(JJ)
   40 CONTINUE
      XC=X(KK)-X(JJ)
      YC=Y(KK)-Y(JJ)
      ZC=Z(KK)-Z(JJ)
      AL=SQRT(XC*XC+YC*YC+ZC*ZC)
      CX(I)=XC/AL
      CY(I)=YC/AL
      CZ(I)=ZC/AL
      XC2 = XLL-X(JJ)
      YC2 = YLL-Y(JJ)
      ZC2 = ZLL-Z(JJ)
C     AL2 = SQRT(XC2*XC2+YC2*YC2+ZC2*ZC2)
C     AL2 IS DISTANCE FROM J NODE TO L NODE
      XC3 = YC*ZC2-ZC*YC2
      YC3 = ZC*XC2-XC*ZC2
      ZC3 = XC*YC2-YC*XC2
      AL3 = SQRT(XC3*XC3+YC3*YC3+ZC3*ZC3)
C     AL3 IS LENGTH OF VECTOR JK CROSS JL
      XC4 = YC3*ZC-YC*ZC3
      YC4 = ZC3*XC-ZC*XC3
      ZC4 = XC3*YC-XC*YC3
      AL4 = SQRT(XC4*XC4+YC4*YC4+ZC4*ZC4)
C     AL4 IS LENGTH OF VECTOR (JK CROSS JL) CROSS JK
C     CX2(I) IS THE COSINE OF THE ANGLE BETWEEN MEMBER(I) Y AXIS
C           AND STRUCTURE X AXIS
C     CY2(I) IS THE COSINE OF THE ANGLE BETWEEN MEMBER(I) Y AXIS
C           AND STRUCTURE Y AXIS. ETC.
C     CX3(I) IS THE COSINE OF THE ANGLE BETWEEN MEMBER(I) Z AXIS
C           AND STRUCTURE X AXIS
      CX3(I) = XC3/AL3
      CY3(I) = YC3/AL3
      CZ3(I) = ZC3/AL3
      CX2(I) = XC4/AL4
      CY2(I) = YC4/AL4
      CZ2(I) = ZC4/AL4
      ALEN(I)=AL
      RETURN
      END
      SUBROUTINE FM(WLY,WRY,THTDY,WLZ,WRZ,THTDZ,AL,FEM,
     1 NPX,NPY,NPZ)
      DIMENSION FEM(12)
      THTRY=THTDY/57.3
      THTRZ=THTDZ/57.3
      WLYN=WLY*COS(THTRY)
      WLYPAR=WLY*SIN(THTRY)
      WRYN=WRY*COS(THTRY)
      WRYPAR=WRY*SIN(THTRY)
      WLZN=WLZ*COS(THTRZ)
      WLZPAR=WLZ*SIN(THTRZ)
      WRZN=WRZ*COS(THTRZ)
      WRZPAR=WRZ*SIN(THTRZ)
C     POSITIVE PARALLEL COMPONENT FOR Y LOAD ACTS TOWARD J NODE
C     POSITIVE PARALLEL COMPONENT FOR Z LOAD ACTS TOWARD K NODE
      FEM1Y=0.5*AL*(WLYPAR*0.667+WRYPAR*0.333)
      FEM1Z=-1.*0.5*AL*(WLZPAR*0.667+WRZPAR*0.333)
      FEM(1)=FEM1Y+FEM1Z
      FEM7Y=0.5*AL*(WLYPAR*0.333+WRYPAR*0.667)
      FEM7Z=-1.*0.5*AL*(WLZPAR*0.333+WRZPAR*0.667)
      FEM(7)=FEM7Y+FEM7Z
      IF(NPX.EQ.2.OR.NPX.EQ.4)FEM(7)=FEM(7)+FEM(1)
      IF(NPX.EQ.2.OR.NPX.EQ.4)FEM(1)=0.
      IF(NPX.EQ.3.OR.NPX.EQ.5)FEM(1)=FEM(1)+FEM(7)
      IF(NPX.EQ.3.OR.NPX.EQ.5)FEM(7)=0.
      FEM(4)=0.
      FEM(10)=0.
      IF(NPZ.EQ.1)GO TO 100
      FEM(6)=-1./12.*WLYN*AL**2-1./30.*(WRYN-WLYN)*AL**2
      FEM(12)=1./12.*WLYN*AL**2+1./20.*(WRYN-WLYN)*AL**2
      IF(NPZ.EQ.0)GO TO 110
      IF(NPZ.EQ.3)GO TO 90
C     J END PINNED FOR STRONG AXIS BENDING
      FEM(12)=FEM(12)-0.5*FEM(6)
      FEM(6)=0.
      GO TO 110
   90 CONTINUE
C     K END PINNED FOR STRONG AXIS BENDING
      FEM(6)=FEM(6)-0.5*FEM(12)
      FEM(12)=0.
      GO TO 110
  100 FEM(6)=0.
      FEM(12)=0.
  110 FEM(2)=-WLYN*AL*.5-(WRYN-WLYN)*AL*.5/3.
     1    +(FEM(12)+FEM(6))/AL
      FEM(8)=-WLYN*AL*.5-(WRYN-WLYN)*AL*.333333
     1    -(FEM(12)+FEM(6))/AL
C     Y AXIS BENDING (WEAK)
      IF(NPY.EQ.1)GO TO 200
      FEM(5)=1./12.*WLZN*AL**2+1./30.*(WRZN-WLZN)*AL**2
      FEM(11)=-1./12.*WLZN*AL**2-1./20.*(WRZN-WLZN)*AL**2
      IF(NPY.EQ.0)GO TO 210
      IF(NPY.EQ.3)GO TO 190
C     J END PINNED FOR WEAK AXIS BENDING
      FEM(11)=FEM(11)-0.5*FEM(5)
      FEM(5)=0.
      GO TO 210
  190 CONTINUE
C     K END PINNED FOR WEAK AXIS BENDING
      FEM(5)=FEM(5)-0.5*FEM(11)
      FEM(11)=0.
      GO TO 210
  200 CONTINUE
      FEM(5)=0.
      FEM(11)=0.
  210 CONTINUE
      FEM(3)=-WLZN*AL*.5-(WRZN-WLZN)*AL*.5/3.
     1     -(FEM(11)+FEM(5))/AL
      FEM(9)=-WLZN*AL*.5-(WRZN-WLZN)*AL*.333333
     1     +(FEM(11)+FEM(5))/AL
      RETURN
      END
C
      SUBROUTINE FM2(PV,PD,AL,FEM,NP,NCOD)
C     SUB TO CALC FEM FOR CONC LOADS
      DIMENSION FEM(12)
C   NCOD EQ 1 INDICATES LOAD IN Z DIRECTION, MOMENTS ABOUT Y AXIS
C   NCOD EQ 0 INDICATES LOAD IN Y DIRECTION, MOMENTS ABOUT Z AXIS
      IF(NCOD.EQ.1)GO TO 50
      FAC=1.
      I1=2
      I2=8
      I3=6
      I4=12
      GO TO 80
   50 FAC=-1.
      I1=3
      I2=9
      I3=5
      I4=11
   80 PDP=AL-PD
      IF(NP.EQ.1)GO TO 100
      F65=-PV*PD*PDP**2/AL**2*FAC
      F1211=PV*PD**2*PDP/AL**2*FAC
      IF(NP.EQ.0)GO TO 110
      IF(NP.EQ.3)GO TO 90
C     J END PINNED
      F1211=F1211-0.5*F65
      F65=0.
      GO TO 110
   90 CONTINUE
C     K END PINNED
      F65=F65-0.5*F1211
      F1211=0.
      GO TO 110
  100 F65=0.
      F1211=0.
  110 FEM(I1)=FEM(I1)-PV*PDP/AL+(F65+F1211)/AL*FAC
      FEM(I2)=FEM(I2)-PV*PD/AL-(F65+F1211)/AL*FAC
      FEM(I3)=FEM(I3)+F65
      FEM(I4)=FEM(I4)+F1211
      RETURN
      END
C
      SUBROUTINE IND(I,INN)
      DIMENSION INN(6)
      IN=6*I-6
      DO 20 J=1,6
   20 INN(J)=IN+J
      RETURN
      END
      SUBROUTINE FORMSM(SM,AL,AX,E,G,RIX,RIY,RIZ,NPX,NPY,NPZ)
C     COMPUTES MEMBER STIFFNESS MATRIX IN LOCAL COORDINATES.
      DIMENSION SM(12,12)
      A = AX*E/AL
C     IF NPX .GT.1, THERE IS AN AXIAL RELEASE
      IF(NPX.GT.1)A=0.
      DO 10 I = 1,12
      DO 10 M = 1,12
 10   SM(I,M) = 0.
      SM(1,1) = A
      SM(7,7) = A
      SM(1,7) = -A
      SM(7,1)=-A
      SM(4,4) =RIX*G/AL
C     IF NPX .EQ.1,2,3, THERE IS A TORSION RELEASE
      IF(NPX.EQ.1.OR.NPX.EQ.2.OR.NPX.EQ.3)SM(4,4)=0.
      SM(10,10)=SM(4,4)
      SM(4,10) = -SM(4,4)
      SM(10,4)=SM(4,10)
      SM(5,5)=0.00001*E*RIY/AL
      SM(11,11)=SM(5,5)
      SM(6,6)=0.00001*E*RIZ/AL
      SM(12,12)=SM(6,6)
C     NPY = 1 MEANS PINNED BOTH ENDS FOR WEAK AXIS BENDING
C     NPZ = 1 MEANS PINNED BOTH ENDS FOR STRONG AXIS BENDING
      IF(NPY.EQ.1.AND.NPZ.EQ.1)RETURN
      CONY=12.
      CONZ=12.
C     IF BOTH ENDS PINNED, CON =0.00001
      IF(NPZ.EQ.1)CONZ=0.00001
      IF(NPY.EQ.1)CONY=0.00001
C     IF EITHER END PINNED, CON = 3.
      IF(NPY.GT.1)CONY=3.
      IF(NPZ.GT.1)CONZ=3.
      YY = RIY*E
      ZZ = RIZ*E
      AL2 = AL*AL
      AL3 = AL2*AL
      C1Z = CONZ*ZZ/AL3
      C1Y = CONY*YY/AL3
      C2Y = 6.*YY/AL2
      C2Z = 6.*ZZ/AL2
      C3Z = 4.*ZZ/AL
      C3Y = 4.*YY/AL
      SM(2,2) = C1Z
      SM(2,8) = -C1Z
      SM(8,8) = C1Z
      SM(3,3) = C1Y
      SM(3,9) = -C1Y
      SM(9,9) = C1Y
C     NPZ = 1 MEANS PINNED BOTH ENDS FOR STRONG AXIS BENDING
      IF(NPZ.EQ.1)GO TO 200
C     NPZ = 2 MEANS PINNED J END FOR STRONG(Z) AXIS BENDING
      IF(NPZ.EQ.2)GO TO 50
C     NPZ = 3 MEANS PINNED K END FOR Z AXIS BENDING
      IF(NPZ.EQ.3) GO TO 100
C     NEITHER AXIS PINNED FOR Z AXIS
      SM(2,6) = C2Z
      SM(2,12) = C2Z
      SM(6,6) = C3Z
      SM(6,8) = -C2Z
      SM(6,12) = 0.5*C3Z
      SM(8,12) = -C2Z
      SM(12,12) = C3Z
      GO TO 200
   50 CONTINUE
C     J END PINNED FOR STRONG AXIS BENDING
      SM(2,12)=0.5*C2Z
      SM(8,12)=-0.5*C2Z
      SM(12,12)=0.75*C3Z
      GO TO 200
  100 CONTINUE
C     K END PINNED FOR STRONG AXIS BENDING
      SM(2,6)=0.5*C2Z
      SM(6,6)=0.75*C3Z
      SM(6,8)=-0.5*C2Z
  200 CONTINUE
C     WEAK AXIS BENDING
      IF(NPY.EQ.1)GO TO 350
      IF(NPY.EQ.2)GO TO 250
      IF(NPY.EQ.3)GO TO 300
C     NEITHER END PINNED FOR WEAK AXIS BENDING
      SM(3,5) = -C2Y
      SM(3,11) = -C2Y
      SM(5,5) = C3Y
      SM(5,9) = C2Y
      SM(5,11) = 0.5*C3Y
      SM(9,11) = C2Y
      SM(11,11) = C3Y
      GO TO 350
  250 CONTINUE
C     J END PINNED FOR WEAK AXIS BENDING
      SM(3,11)=-0.5*C2Y
      SM(9,11)=0.5*C2Y
      SM(11,11)=0.75*C3Y
      GO TO 350
  300 CONTINUE
C     K END PINNED FOR WEAK AXIS BENDING
      SM(3,5)=-0.5*C2Y
      SM(5,5)=0.75*C3Y
      SM(5,9)=0.5*C2Y
  350 CONTINUE
      DO 20 I = 1,11
      J = I+1
      DO 20 M = J,12
   20 SM(M,I) = SM(I,M)
      RETURN
      END
      SUBROUTINE INDX(JJ,KK,IN)
      DIMENSION IN(12)
      J6 = 6*JJ
      DO 20 I = 1,6
      J = 7-I
   20 IN(J) = J6-I+1
      K6 = 6*KK
      DO 30 I = 1,6
      J = 13-I
   30 IN(J) = K6-I+1
      RETURN
      END
      SUBROUTINE MULT(A,B,JB,C)
      DIMENSION A(12,12),B(12,JB),C(12,JB)
      DO 10 I = 1,12
      DO 10 N = 1,JB
      SUM =  0.0
      DO 20 L = 1,12
20    SUM = SUM+A(I,L)*B(L,N)
10    C(I,N) = SUM
      RETURN
      END
      SUBROUTINE FORMRT(J)
      COMMON/BB/CX(600),CY(600),CZ(600),CX2(600),CY2(600),CZ2(600),
     1CX3(600),CY3(600),CZ3(600),ALEN(600),RT(12,12),RTRANS(12,12)
      DIMENSION C(3,3)
      DO 10 I = 1,12
      DO 10 N = 1,12
  10  RT(I,N)=0.
      C(1,1) = CX(J)
      C(1,2) = CX2(J)
      C(1,3) = CX3(J)
      C(2,1) = CY(J)
      C(2,2) = CY2(J)
      C(2,3) = CY3(J)
      C(3,1) = CZ(J)
      C(3,2) = CZ2(J)
      C(3,3) = CZ3(J)
      INDA = 0
      DO 30 I = 1,12
      IF(I.EQ.4.OR.I.EQ.7.OR.I.EQ.10)INDA = INDA+3
      INDB = 1
      IF(I.EQ.2.OR.I.EQ.5.OR.I.EQ.8.OR.I.EQ.11)INDB = 2
      IF(I.EQ.3.OR.I.EQ.6.OR.I.EQ.9.OR.I.EQ.12)INDB =3
      DO 30 JJ = 1,3
      K = JJ+INDA
  30  RT(K,I) = C(INDB,JJ)
C     FORM RT(TRANSPOSE)
      DO 31 I = 1,12
      DO 31 N = 1,12
  31  RTRANS(N,I) = RT(I,N)
      RETURN
      END
C
      SUBROUTINE KL(N6,NMEM,JDIM)
      COMMON/AA/X(400),Y(400),Z(400),JEND(600),KEND(600),LEND(600),
     1 BJT(2400),NJXT(10),SXT(10),NJYT(10),SYT(10),NJZT(10),SZT(10)
      COMMON/CC/A(70000),AJ(2400),IDIAG(2400),KLIM(2400),IB,NDIAG
      COMMON/KK/NN(400)
      DIMENSION IN(12)
C     ALGORITHM TO COMPUTE KLIM, IDIAG VECTORS.
C     LOOP THROUGH ALL MEMBERS, COMPUTE PSUEDO 
C         INDEX VECTOR FOR EACH WITH J .LT. KL
C     COMPUTE COLUMN HEIGHT TO SKYLINE.
C     IB = HALFBANDWIDTH (INCLUDING DIAGONAL).
C     COLUMN ENTRIES NUMBERED FROM TOP DOWN TO DIAGONAL
      DO 10 I = 1,N6
  10  KLIM(I) = 0
      IB = 0
      DO 20 M = 1,NMEM
      J = NN(JEND(M))
      K = NN(KEND(M))
      IF(J .LT. K) GOTO 25
      J = K
      K = NN(JEND(M))
C
C     ASSUME K .GT. J SINCE S IS INDEPENDENT OF JEND, KEND
C         NUMBERING USED.
  25  J6 = 6*J
      DO 200 I = 1,6
      J = 7-I
  200 IN(J) = J6-I+1
      K6 = 6*K
      DO 300 I = 1,6
      J = 13-I
  300 IN(J) = K6-I+1
C
C     COMPUTE COLUMN HEIGHTS FOR MTH MEMBER.
C
      DO 30 L = 1,12
      N = IN(L)
      IH = IN(L)-IN(1)+1
      IF(KLIM(N).GT.IH)GO TO 30
      KLIM(N) = IH
      IF(IB.LT.IH)IB = IH
  30  CONTINUE
  20  CONTINUE
      NDIAG = 0
      DO 40 I = 1,N6
C      IDIAG(I) = NDIAG+1  USE THIS FOR BOTTOM UP NUMBERING
      IDIAG(I)=NDIAG+KLIM(I)
      NDIAG = NDIAG+KLIM(I)
  40  CONTINUE
      WRITE(6,900) NDIAG,IB
  900 FORMAT(/,1X,27H NO. OF TERMS IN A MATRIX =,I6,
     119H   HALF BAND WIDTH=,I5,/)
      IF(NDIAG.LE.JDIM)GO TO 50
      WRITE(6,910)JDIM
  910 FORMAT(39H NO. TERMS EXCEEDS DIMENSIONED LIMIT OF,I6)
      STOP
  50  RETURN
      END
C
      SUBROUTINE UDU2(LU,NUMEQ)
      COMMON/CC/A(70000),X(2400),IDIAG(2400),KLIM(2400),IB,NDIAG
      NEM1=NUMEQ-1
      DO 450 I=1,NEM1
      IDIAGI=IDIAG(I)
      IP1=I+1
      DO 440 J=IP1,NUMEQ
      J1=IDIAG(J)-(J-I)
      IF(J1.LE.IDIAG(J-1)) GO TO 440
      IF(LU.EQ.0) GO TO 435
      FAC=A(J1)/A(IDIAGI)
      DO 430 K=J,NUMEQ
      K1=IDIAG(K)-(K-I)
      IF(K1.LE.IDIAG(K-1)) GO TO 430
      K2=IDIAG(K)-(K-J)
      A(K2)=A(K2)-A(K1)*FAC
  430 CONTINUE
      A(J1)=FAC
  435 CONTINUE
      X(J)=X(J)-X(I)*A(J1)
  440 CONTINUE
  450 CONTINUE
      IROW=NUMEQ+1
      DO 470 I=1,NUMEQ
      IROW=IROW-1
      IDIAGI=IDIAG(IROW)
      X(IROW)=X(IROW)/A(IDIAGI)
      IF(IROW.EQ.NUMEQ) GO TO 470
      IP1=IROW+1
      DO 460 J=IP1,NUMEQ
      J1=IDIAG(J)-(J-IROW)
      IF(J1.LE.IDIAG(J-1)) GO TO 460
      X(IROW)=X(IROW)-A(J1)*X(J)
  460 CONTINUE
  470 CONTINUE
      RETURN
      END
